package dev.fyloz.trial.colorrecipesexplorer.services;

import dev.fyloz.trial.colorrecipesexplorer.builders.CompanyBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityLinkedException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.dao.CompanyDao;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.*;

@ExtendWith(SpringExtension.class)
@SpringBootTest
public class CompanyServiceTest extends AbstractServiceTest<Company, CompanyService, CompanyDao> {

    @MockBean
    private CompanyDao companyDao;

    @Autowired
    @SpyBean
    private CompanyService companyService;

    @MockBean
    private RecipeService recipeService;

    @BeforeEach
    public void setUp() {
        companyService.setCompanyDao(companyDao);
        companyService.setRecipeService(recipeService);

        super.init(companyService, companyDao);

        Company company = getEntityDifferentName();

        when(companyDao.existsByName(anyString())).thenReturn(false);
        when(companyDao.existsByName(company.getName())).thenReturn(true);

        when(recipeService.existsByCompany(any())).thenReturn(false);
        when(recipeService.existsByCompany(company)).thenReturn(true);
    }


    @Test
    public void whenExistsByName_thenReturnTrue() {
        // given
        Company company = getEntityDifferentName();

        // when
        boolean found = companyService.existsByName(company.getName());

        // then
        assertTrue(found);
    }

    @Test
    public void whenExistsByName_thenReturnFalse() {
        // given
        Company company = getEntity();

        // when
        boolean found = companyService.existsByName(company.getName());

        // then
        assertFalse(found);
    }

    @Test
    public void whenIsLinkedToRecipes_thenReturnTrue() {
        // given
        Company company = getEntityDifferentName();

        // when
        boolean found = companyService.isLinkedToRecipes(company);

        // then
        assertTrue(found);
    }

    @Test
    public void whenIsLinkedToRecipes_thenReturnFalse() {
        // given
        Company company = getEntity();

        // when
        boolean found = companyService.isLinkedToRecipes(company);

        // then
        assertFalse(found);
    }

    @Test
    @Override
    public void whenSave_thenReturnEntity() {
        // given
        Company company = getEntityDifferentId();

        // when
        Company found = companyService.save(company);

        // then
        assertEquals(company, found);
    }

    @Test
    @Override
    public void whenSave_thenThrowEntityAlreadyExistsException() {
        // given
        Company company = getEntityDifferentName();

        // when
        EntityAlreadyExistsException thrown = assertThrows(EntityAlreadyExistsException.class, () -> companyService.save(company));

        // then
        assertEquals(thrown.getIdentifierType(), ModelException.IdentifierType.NAME);

        super.whenSave_thenThrowEntityAlreadyExistsException();
    }

    @Test
    public void whenDelete_thenThrowEntityLinkedException() {
        // given
        Company company = getEntityDifferentName();

        // then
        assertThrows(EntityLinkedException.class, () -> companyService.delete(company));
    }

    @Override
    protected Company getEntity() {
        return new CompanyBuilder().build();
    }

    @Override
    protected Company getEntityDifferentId() {
        return new CompanyBuilder()
                .id(1L)
                .build();
    }

    @Override
    protected Company getEntityNullId() {
        return new CompanyBuilder()
                .id(null)
                .build();
    }

    private Company getEntityDifferentName() {
        return new CompanyBuilder()
                .name("Design")
                .build();
    }
}
